;generator kolisaveho tonu pro plaseni hlodavcu
;autor: Ivan Stupka, (c) SISIE
;

;V1,26.10.2012

	list n=0
	errorlevel -302
	errorlevel -306

	processor 16F676
;Fosc=4.00 MHz, vnitrni oscilator

	#include ".\include\P16F676.INC"

;noexpand

;------- FLASH --------------------
;rozsah: 0 - 0x3ff

;------------------
;obsazeni registru
;==================
;globalni: ~RAPU(OPTION.7)=0, ANSEL=0, CMCON=0x7

;porta
_tst2	equ 0	;INpu	- (13), test-F/2, proti (14)-GND, ICSP data
_tst4	equ 1	;INpu	- (12), test-F/4, proti (14)-GND, ICSP clock
_ra2	equ 2	;INpu	- (11), nepouzito
_ra3	equ 3	;IN	- (4),  100K k GND, ICSP VPP
_ra4	equ 4	;INpu	- (3), nepouzito
_ra5	equ 5	;INpu	- (2), nepouzito
;porta	=	0xff
;trisa	=	0xff, default
;wpua	=	0xff, default
;ioca	=	0, default
;-----
;portc
;piny RC0-RC2 paralelne, RC3-RC5 tez, vystup pro piezorepro
;portc	=	0x0
;trisc	=	0x0

;
;****************** popis uzivatelskych registru ***************************
;rozsah 0x20 - 0x5f, zrcadli se v obou bancich
tstmsk	equ	0x20	;jen jeden bit v 1: 0=norm, 1=tst2, 2=tst4
crc	equ	0x21	;pseudorandom generator


tbase2	equ	0x24	;hlavni timer, hi
tbase1	equ	0x25	;hlavni timer, mid
tbase0	equ	0x26	;hlavni timer, lo




;============================================================================
	org	0

reset:
	clrf	portc		;aby se piny nepraly navzajem
	bsf	status,rp0	;Bank 1
;nejdriv kalibrace oscilatoru
	call	0x3ff		;Get the cal value
	movwf	osccal		;Calibrate
	clrf	option_reg	;wpu en,tmr0 instr w.prescaler 1:2, tj.wdtout=18ms
;a init portu
	movlw	0xff
	movwf	trisa
	movwf	wpua
	clrf	trisc
	clrf	ansel		;vse digitalni
	bcf	status,rp0	;Bank 0
	movwf	porta
	movlw	7
	movwf	cmcon		;off, piny digitalni
;init user reg
	clrf	tbase0
	clrf	tbase1
	clrf	tbase2
	movlw	1		;nesmi byt 0
	movwf	crc
	movwf	tstmsk
	goto	loop1
;====================================================
;
;============== hlavni program ================

;algoritmus:
;kmitocet piskani: zakladni rozsah  52T + (0..7) =  59T, stred 56T(17.9kHz)
;prvni polovina kmitu trva vzdy 28T, druha 24-31T
;s periodou 1s je rozmitan kmitocet o +-2T, tj. max/min=58/54= cca 1 pulton,
;tj. max rozsah je 50T (20.0kHz) - 61T (16.4kHz), 28/22-33T
;
;po nahodne delce trvani M (1..8)*16 vterinovych cyklu se zaradi
;nahodna pauza o delce (1..4)s s pravdepodobnosti 50%
;pote se zmeni kmitocet o 0..7 T
;
;nahodna cisla jsou z pseudorandom generatoru o delce 8 bit s ridicim polynomem
;X8 + X5 + X4 + 1, vzor: if(c & 1) c=(c >> 1) ^ 0x8c; else c= c >> 1;
;pouzite bity z crc:
;posun kmitoctu piskani = <210>,
;delka trvani piskani <432> xor <076>,
;pauza po skonceni cyklu <7> & (<43> xor <32>)
;
;hlavni citac:
;byte---2---- ---1---- ---0----
;bit 76543210 76543210 76543210
;                  xxx xxxxxxxx  citac kmitu 2^11 * 58T = 114688us = cykl C/8
;               xxx              vibrato posuv: 1,3=+1, 5,7=-1, 2=+2, 6=-2, 0,4=0
;          xx xx                 pocet C (917504 us, cca 1 s) cca 14.7s=M
;    111xxx                      random nasobky M





;zacatek smycky, prechod z konce druhe pulperiody v T=16 (pro Tper=22)
loop:
;zapis do portu
	rlf	tbase0,w	;cy=0, kvuli testovani pro nizsi kmitocty
	andwf	tstmsk,w
	movlw	0x7		;hi 3 bity v 0, lo v 1
	skpz
	movlw	0x38
	movwf	portc		;hi 3 bity v 1, lo v 0
;T=22 (max T=33), hotovo
;--------------  prvni pulperioda -------------------
;zacatek smycky, trva 28T, T=0
	clrwdt
;posun hlavniho timeru
	movlw	1
	addwf	tbase0,f
	skpnc
	addwf	tbase1,f
	skpnc
	addwf	tbase2,f	;cy=1 znamena konec cyklu
	skpc
	goto	loop3		;pouze cekame, T=10
;cyklus ukoncen, nastavime novy, a testujeme pauzu
	btfsc	crc,7		;ma byt pauza?
	goto	loop4		;ano
;T=11, nastaveni M a CRC
loop1:
	swapf	crc,w		;nove M
	xorwf	crc,w
	andlw	0x1c		;<432> xor <076>
	iorlw	0xe0		;hi bity v 1
	movwf	tbase2
	clrc			;novy CRC
	rrf	crc,w
	skpnc
	xorlw	0x8c
	movwf	crc
;T=21, zapis do portu
loop2:
	setc			;flag prvni pulperiody
	rlf	tbase0,w	;cy=1, kvuli testovani pro nizsi kmitocty
	andwf	tstmsk,w
	movlw	0x7		;hi 3 bity v 0, lo v 1
	skpz
	movlw	0x38
	movwf	portc		;hi 3 bity v 1, lo v 0
;T=28, hotovo
;--------------  druha pulperioda -------------------
;T=0
;----- promenne zpozdeni, trva 14-25T, 14 + (0..11)
;urceni zpozdeni dle vibrata trva 10-14 T
	movlw	0		;priprava pozice 0, tj. offset=0
	btfsc	tbase1,4
	movlw	2		;priprava pozice 2, tj. offset=2
	btfsc	tbase1,3	;byla 0 nebo 2, tj. suda?
	movlw	1		;ne, je licha, 1 nebo 3, tj. offset=1
	btfsc	tbase1,5	;druha pulperioda, tj. pozice 4-7?
	sublw	0		;ano, zaporny posuv
	addlw	2		;do rozsahu 4-0
	addwf	pcl,f		;a hned zpozdeni...
	nop
	nop
	nop
	nop
;zpozdeni offsetu, trva 4-11T
	movf	crc,w
	andlw	0x7		;pouze zajimave bity
	addwf	pcl,f		;cy=0
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;T=14 pro dobu pulperiody 22T, zbyva 8T
	goto	loop		;v case T=16
;---------------- cekaci vypln ----------------------
;T=10, vraci se v T=21
loop3:
;kontrola zadosti o test
	movlw	1		;zakladni kmitocet
	btfss	porta,_tst2
	movlw	2		;polovicni kmitocet
	btfss	porta,_tst4
	movlw	4		;ctvrtinovy kmitocet
	movwf	tstmsk
	goto	$+1
	nop
	goto	loop2		;T=21
;---------------- pauza ----------------------
loop4:
	rrf	crc,w
	xorwf	crc,w
	andlw	0xc		;4*(0-3)
	addlw	4		;4*(1-4)
	movwf	tbase2
loop7:
	clrwdt
	decfsz	tbase0,f
	goto	loop7		;vnitrni cyklus trva 4*256T=1024T
	decfsz	tbase1,f
	goto	loop7		;stredni cyklus trva 256*1024T=262144us ~.25s
	decfsz	tbase2,f
	goto	loop7		;vnejsi cyklus trva 0.25*4*(1-4) sec
	goto	loop1
;=================================================================


;=====================================================================
;
;
;	********************
;	*  konec programu  *
;	********************
;
;
;================================================

;kalibrace oscilatoru
	org	0x3ff
	retlw	0x80		;melo by byt prepsano programatorem


;konfiguracni slova
	org	0x2007

;config:
;13 12 11 10 9 8 7 6 5 4 3 2 1 0
;                          1 0 0 int osc, piny as IO
;                        1       WDTE en
;                      1         PWRT dis
;                    0		 MCLR dis
;                  1		 BODEN enabled
;              1 1               no code & data protection
;       0  0 0                   nepouzito
; 1  0                           bandgap, mel byt doplnen programatorem
;
	dw	0x21dc


	end





