;generator kolisaveho tonu pro plaseni hlodavcu
;autor: Ivan Stupka, (c) SISIE
;

;V2, 03.01.2016 - prepsano pro PIC16F505
;V1, 26.10.2012 - psano pro PIC16F676

	list n=0
	errorlevel -302
	errorlevel -306

	processor 16F505
;Fosc=4.00 MHz, vnitrni oscilator

	#include ".\include\P16F505.INC"

;noexpand

;------- FLASH --------------------
;rozsah: 0 - 0x3ff

;------------------
;obsazeni registru
;==================

;portb
_tst2	equ 0	;INpu	- (13), test-F/2, proti (14)-GND, ICSP data
_tst4	equ 1	;INpu	- (12), test-F/4, proti (14)-GND, ICSP clock
_rb2	equ 2	;IN	- (11), nepouzito
_rb3	equ 3	;INpu	- (4),  ICSP VPP
_rb4	equ 4	;INpu	- (3), nepouzito
_rb5	equ 5	;IN	- (2), nepouzito
;portb	=	0xff
;trisb	=	0xff, default
;v provozu spojit _rb2 s _tst4 a _rb4 s _rb5 aby nebylo nutno pouzit externi pullup odpory
;-----
;portc
;piny RC0-RC2 paralelne, RC3-RC5 tez, vystup pro piezorepro
;portc	=	0x0
;trisc	=	0x0

;
;****************** popis uzivatelskych registru ***************************
;rozsah 0x08 - 0x0f, zrcadli se ve vsech bancich
tstmsk	equ	0x08	;jen jeden bit v 1: 0=norm, 1=tst2, 2=tst4
crc	equ	0x09	;pseudorandom generator


tbase2	equ	0x0d	;hlavni timer, hi
tbase1	equ	0x0e	;hlavni timer, mid
tbase0	equ	0x0f	;hlavni timer, lo




;============================================================================
	org	0

reset:
	movwf	osccal
	movlw	0xff
	tris	portb		;vsechno IN
	movwf	portb
	movlw	0		;vsechno OUT
	movwf	portc		;aby se piny nepraly navzajem
	tris	portc
	option			;pullup en, WDTper=18ms
;init user reg
	clrf	tbase0
	clrf	tbase1
	movlw	1		;nesmi byt 0
	movwf	crc
	movwf	tstmsk
	goto	loop1
;====================================================
;
;============== hlavni program ================

;algoritmus:
;kmitocet piskani: zakladni rozsah  52T + (0..7) =  52 - 59T
;prvni polovina kmitu T1h trva vzdy 28T, druha T2h = 24-31T
;s periodou 1s je rozmitan kmitocet o +-2T, tj. max/min=58/54= cca 1 pulton
;tj. celkovy rozsah tonu je min 28+22 = 50T (20.0kHz) max 28+33 = 61T (16.4kHz)
;
;po nahodne delce trvani M (1..8)*16 vterinovych cyklu se zaradi
;nahodna pauza o delce (1..4)s s pravdepodobnosti 50%
;pote se zmeni kmitocet o 0..7 T
;
;nahodna cisla jsou z pseudorandom generatoru o delce 8 bit s ridicim polynomem
;X8 + X5 + X4 + 1, vzor: if(c & 1) c=(c >> 1) ^ 0x8c; else c= c >> 1;
;pouzite bity z crc:
;posun kmitoctu piskani = <210>,
;delka trvani piskani <432> xor <076>,
;pauza po skonceni cyklu <7> & (<43> xor <32>)
;
;hlavni citac:
;byte---2---- ---1---- ---0----
;bit 76543210 76543210 76543210
;                  xxx xxxxxxxx  citac kmitu 2^11 * 58T = 114688us = cykl C/8
;               xxx              vibrato posuv sekvence: -2,-1,0,1,2,1,0,-1
;          xx xx                 pocet C (917504 us, cca 1 s), tj. M = 16*C = 14.7s
;    111xxx                      random nasobky M

;=======================================================================
;
;zacatek smycky, prechod z konce druhe pulperiody o 7 T driv, nez je delka druhe poloviny kmitu,
;tj. v T=15..26 (pro T2h=22..33)
loop:
;zapis do portu
	clrc			;flag druhe pulperiody pro zakladni kmitocet,
	rlf	tbase0,w	;bit0=0, dalsi bity testu pro nizsi kmitocty
	andwf	tstmsk,w
	movlw	0x7		;hi 3 bity v 0, lo v 1
	skpz
	movlw	0x38
	movwf	portc		;hi 3 bity v 1, lo v 0
;min T=22, max T=33, hotovo
;--------------  prvni pulperioda -------------------
;zacatek smycky, trva T1h = 28T, nyni T=0
	clrwdt
;posun hlavniho timeru
	movlw	1
	addwf	tbase0,f
	skpnc
	addwf	tbase1,f
	skpnc
	addwf	tbase2,f	;cy=1 znamena konec cyklu
	skpc
	goto	loop5		;pouze cekame, T=10
;cyklus ukoncen, nastavime novy, a testujeme pauzu
	btfsc	crc,7		;ma byt pauza?
	goto	loop6		;ano
;T=11, nastaveni M a CRC
loop1:
	swapf	crc,w		;nove M
	xorwf	crc,w
	andlw	0x1c		;<432> xor <076>
	iorlw	0xe0		;hi bity v 1
	movwf	tbase2
	clrc			;novy CRC
	rrf	crc,w
	skpnc
	xorlw	0x8c
	movwf	crc
;T=21, zapis do portu
loop2:
	setc			;flag prvni pulperiody pro zakladni kmitocet,
	rlf	tbase0,w	;bit0=1, dalsi bity testu pro nizsi kmitocty
	andwf	tstmsk,w
	movlw	0x7		;hi 3 bity v 0, lo v 1
	skpz
	movlw	0x38
	movwf	portc		;hi 3 bity v 1, lo v 0
;T=28, hotovo
;--------------  druha pulperioda -------------------
;
;----- promenne zpozdeni, trva 15-26T, 15 + (0..11)
;T=0, zpozdeni offsetu dle crc<2..0> 0-7 T, trva 8-15T
	btfsc	crc,0
	goto	$+1		;t = 2/3, dT=1 pro bit 0
	btfsc	crc,1
	goto	$+3
	nop
	goto	$+1		;t = 3/5, dT=2 pro bit 1
	btfss	crc,2
	goto	$+4
	nop
	goto	$+1
	goto	$+1		;t = 3/7, dT=4 pro bit 2
;T = 8-15
;zpozdeni vibrata dle tbase1<5..3>, trva 7 - 11 T
;sekvence zpozdeni je trojuhelnikova, tj. 0,1,2,3,4,3,2,1
	btfsc	tbase1,5
	goto	loop3		;>=4, tj. klesajici sekvence
	btfsc	tbase1,3
	goto	$+1		;t=4/5, dT=1 pro bit 3
	btfss	tbase1,4
	goto	loop		;000:dT=7, 001:dT=8
	nop
	goto	loop		;010:dT=9, 011:dT=10
loop3:
	btfss	tbase1,3
	goto	$+1		;t=5/6, dT=1 pro bit 3
	btfsc	tbase1,4
	goto	loop		;110:dT=9, 111:dT=8
	nop
	goto	loop		;100:dT=11, 101:dT=10
;---------------- cekaci vypln ----------------------
;T=10, vraci se v T=21
loop5:
;kontrola zadosti o test
	movlw	1		;zakladni kmitocet
	btfss	portb,_tst2
	movlw	2		;polovicni kmitocet
	btfss	portb,_tst4
	movlw	4		;ctvrtinovy kmitocet
	movwf	tstmsk
	goto	$+1
	nop
	goto	loop2		;T=21
;---------------- pauza ----------------------
loop6:
	rrf	crc,w
	xorwf	crc,w
	andlw	0xc		;4*(0-3)
	movwf	tbase2
	movlw	4
	addwf	tbase2,f	;4*(1-4)
loop7:
	clrwdt
	decfsz	tbase0,f
	goto	loop7		;vnitrni cyklus trva 4*256T=1024T
	decfsz	tbase1,f
	goto	loop7		;stredni cyklus trva 256*1024T=262144us ~.25s
	decfsz	tbase2,f
	goto	loop7		;vnejsi cyklus trva 0.25*4*(1-4) sec
	goto	loop1
;=================================================================


;=====================================================================
;
;
;	********************
;	*  konec programu  *
;	********************
;
;
;================================================

;kalibrace oscilatoru
;;	org	0x3ff
;;	movlw	0xff		;melo by byt prepsano programatorem


;konfiguracni slovo
	org	0xfff

;config:
;11 10 9 8 7 6 5 4 3 2 1 0
;                    1 0 0 int osc, pin RB4 as IO
;                  1       WDTE en
;                1         CodeProtect off
;              0	   MCLR dis
; 0  0 0 0 0 0

	dw	0x1c


	end





